/*
 * FreeRTOS Kernel V10.2.1
 * Copyright (C) 2017 Amazon.com, Inc. or its affiliates.  All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * http://www.FreeRTOS.org
 * http://aws.amazon.com/freertos
 *
 * 1 tab == 4 spaces!
 */


/*****************************************************************************
 *
 * See the following URL for configuration information.
 * http://www.freertos.org/FreeRTOS-Plus/FreeRTOS_Plus_TCP/TCP_IP_Configuration.html
 *
 *****************************************************************************/

#ifndef FREERTOS_IP_CONFIG_H
#define FREERTOS_IP_CONFIG_H

#include "FreeRTOS.h"
#include "list.h"
#include "semphr.h"

//ӰTCP/IPջִΪĳ
#define ipconfigIP_TASK_PRIORITY          ( configMAX_PRIORITIES - 1 )  //TCP/IPȼ
#define ipconfigIP_TASK_STACK_SIZE_WORDS	256  //TCP/IPĶջС
#define ipconfigUSE_NETWORK_EVENT_HOOK    0  // ¼ Ӻ
/* ¼ٱ绺5,¼Ӧó͵IPջ,һŶӵȴ¼ */
#define ipconfigEVENT_QUEUE_LENGTH		    ( ipconfigNUM_NETWORK_BUFFER_DESCRIPTORS + 5 )

//,ٺͼ¼
#define vLoggingPrintf  printf  //Ϊprintf

#define ipconfigHAS_DEBUG_PRINTF	  0
#if( ipconfigHAS_DEBUG_PRINTF == 1 )
	#define FreeRTOS_debug_printf(X)	vLoggingPrintf X
#endif

#define ipconfigHAS_PRINTF			    0
#if( ipconfigHAS_PRINTF == 1 )
	#define FreeRTOS_printf(X)			  vLoggingPrintf X
#endif

#define ipconfigTCP_MAY_LOG_PORT(xPort)  ( ( xPort ) != 23u )  //:Ϊ23˿־
#define ipconfigCHECK_IP_QUEUE_SPACE     8  //Ӧó͵IPջ,һŶӵȴ¼
#define ipconfigWATCHDOG_TIMER()             //IPÿεõĺ,ι

//Ӳض
#define ipconfigBYTE_ORDER  pdFREERTOS_LITTLE_ENDIAN     //С
#define ipconfigDRIVER_INCLUDED_TX_IP_CHECKSUM       0   //Ӳipv4IPTCP/UDP/ICMPУ
#define ipconfigDRIVER_INCLUDED_RX_IP_CHECKSUM       1   //ӲԶУipv4IPTCP/UDP/ICMPУ
#define ipconfigETHERNET_DRIVER_FILTERS_FRAME_TYPES	 1   //ӲMAC
//#define ipconfigETHERNET_DRIVER_FILTERS_PACKETS
#define ipconfigNETWORK_MTU                          1500
#define ipconfigNUM_NETWORK_BUFFER_DESCRIPTORS		   4  //64  //TCP/IPջ绺(H7ģֵ)
#define ipconfigFILTER_OUT_NON_ETHERNET_II_FRAMES    1   //̫IIʽ̫֡
#define ipconfigUSE_LINKED_RX_MESSAGES               1
#define ipconfigZERO_COPY_TX_DRIVER                  0   //Ͳʹ0Ʒ
#define ipconfigZERO_COPY_RX_DRIVER                  0   //ղʹ0Ʒ
//#define ipconfigBUFFER_PADDING
//#define ipconfigPACKET_FILLER_SIZE

//TCPض
#define ipconfigUSE_TCP				            1     //TCP
#define ipconfigTCP_TIME_TO_LIVE		      128   //TCP TTL
#define ipconfigTCP_TX_BUFFER_LENGTH      1024  //ÿTCP׽ֶһڽյĻһڴĵ
#define ipconfigTCP_RX_BUFFER_LENGTH      1024  //ԽԽ,ҪMCU RAM
#define ipconfigUSE_TCP_WIN			          0     //ʹû(λΪipconfigTCP_MSS)
#define ipconfigTCP_WIN_SEG_COUNT		      4  //64    //(TCP)
#define ipconfigUSE_TCP_TIMESTAMPS        0     //ʹTCPʱ
#define ipconfigTCP_MSS                   ( ipconfigNETWORK_MTU - ipSIZE_OF_IPv4_HEADER - ipSIZE_OF_TCP_HEADER )
#define ipconfigTCP_KEEP_ALIVE				    1     //
#define ipconfigTCP_KEEP_ALIVE_INTERVAL		20    //ʱ(λ )
#define ipconfigTCP_HANG_PROTECTION			  1     //TCP hang protection
#define ipconfigTCP_HANG_PROTECTION_TIME	30    //ʱ(λ )
#define ipconfigIGNORE_UNKNOWN_PACKETS    0     //д0

//UDPض
#define ipconfigUDP_TIME_TO_LIVE		           128  //UDP TTL
#define ipconfigUDP_MAX_SEND_BLOCK_TIME_TICKS  pdMS_TO_TICKS(5000)  //Ϊͳʱʱ
#define ipconfigUDP_MAX_RX_PACKETS             8    //UDP׽ֵRxпԴڵݰ(ÿUDP)

//Ӱ׽Ϊĳ
#define ipconfigINCLUDE_FULL_INET_ADDR	         0  //д0!!!
#define ipconfigALLOW_SOCKET_SEND_WITHOUT_BIND   1  //ͻԶ󶨶˿
#define ipconfigSOCK_DEFAULT_RECEIVE_BLOCK_TIME  pdMS_TO_TICKS(5000)
#define	ipconfigSOCK_DEFAULT_SEND_BLOCK_TIME	   pdMS_TO_TICKS(5000)
#define ipconfigSOCKET_HAS_USER_SEMAPHORE        0  //0
#define ipconfigSUPPORT_SIGNALS                  0  //socketֱӷź FreeRTOS_SignalSocket() ʹһ
#define ipconfigSUPPORT_SELECT_FUNCTION				   0  //ʹFreeRTOS_select

//ӰARPΪĳ
#define ipconfigARP_CACHE_ENTRIES		         4  //32     //ARPпԴڵĿ
#define ipconfigMAX_ARP_RETRANSMISSIONS      8      //ARP
#define ipARP_TIMER_PERIOD_MS                10000  //
#define ipconfigMAX_ARP_AGE			             30     //ARPʱ(λ ipARP_TIMER_PERIOD_MS)
#define ipconfigARP_STORES_REMOTE_ADDRESSES  1
//#define ipconfigUSE_ARP_REVERSED_LOOKUP
//#define ipconfigUSE_ARP_REMOVE_ENTRY

//ӰDHCPƷΪĳ
#define ipconfigUSE_DNS                     0   //DNS
#define ipconfigDNS_USE_CALLBACKS           0   //DNSص
#define ipconfigDNS_REQUEST_ATTEMPTS		    5   //DNS
#define ipconfigUSE_DNS_CACHE				        0   //DNS,ʹûصʹ
#define ipconfigDNS_CACHE_ENTRIES			      8   //DNSĿ
#define ipconfigDNS_CACHE_NAME_LENGTH		    64  //DNS
#define ipconfigUSE_LLMNR                   0
#define ipconfigUSE_NBNS                    0
#define ipconfigUSE_DHCP                    1   /*  */
extern  int ipconfigUSE_DHCP_Variable;          /* ipconfigUSE_DHCP_Variable */
#define ipconfigMAXIMUM_DISCOVER_TX_PERIOD  pdMS_TO_TICKS(5 * 60 * 1000)  //DHCP5min
#define ipconfigDHCP_FALL_BACK_AUTO_IP      1   //DHCPʧʹԶIP
#define ipconfigUSE_DHCP_HOOK               0   //DHCPӺ
#define ipconfigDHCP_REGISTER_HOSTNAME      1   //DHCPע

//ӰIPICMPΪĳ
#define ipconfigCAN_FRAGMENT_OUTGOING_PACKETS  1  //ְַ֧
#define ipconfigREPLY_TO_INCOMING_PINGS				 1  //pingӦ
#define ipconfigSUPPORT_OUTGOING_PINGS				 0  //FreeRTOS_SendPingRequest

//
#define ipconfigHAS_INLINE_FUNCTIONS  1  //Ϊ1()

#ifndef portINLIN  //portmacro.hѾ
  #define portINLINE __inline  //
#endif

//
#define ipconfigIS_VALID_PROG_ADDRESS(x) ( (x) != NULL )


#endif /* FREERTOS_IP_CONFIG_H */












