#include "mDriver/mUSART1.h"
#include "m4G_PPPOS/m4G_PPPOS.h"

static struct {
  u16 read;
  u16 recv;
  
  #define M_U1_DMA_RX_BUF_SIZE  (1024 * 3)
  u8 buf[M_U1_DMA_RX_BUF_SIZE];
} mU1_DMA_RxStruct;

static MyBool mU1_DMA_TxBusy = BOOL_FALSE;

u16 mU1_ReadData(u8 buf[], u16 bufSiz)
{
  if(mU1_DMA_RxStruct.read == mU1_DMA_RxStruct.recv)
  {
    return 0;
  }
  
  u16 ret = 0;
  if(mU1_DMA_RxStruct.read < mU1_DMA_RxStruct.recv)
  {
    ret = mU1_DMA_RxStruct.recv - mU1_DMA_RxStruct.read;
    ret = (bufSiz <= ret) ? bufSiz : ret;
    memcpy(buf, mU1_DMA_RxStruct.buf + mU1_DMA_RxStruct.read, ret);
  }
  else
  {
    ret = mU1_DMA_RxStruct.recv + M_U1_DMA_RX_BUF_SIZE - mU1_DMA_RxStruct.read;
    ret = (bufSiz <= ret) ? bufSiz : ret;
    if(mU1_DMA_RxStruct.read + ret <= M_U1_DMA_RX_BUF_SIZE)
    {
      memcpy(buf, mU1_DMA_RxStruct.buf + mU1_DMA_RxStruct.read, ret);
    }
    else
    {
      u16 tmp = M_U1_DMA_RX_BUF_SIZE - mU1_DMA_RxStruct.read;
      memcpy(buf, mU1_DMA_RxStruct.buf + mU1_DMA_RxStruct.read, tmp);
      memcpy(buf + tmp, mU1_DMA_RxStruct.buf, ret - tmp);
    }
  }
  
  mU1_DMA_RxStruct.read = (mU1_DMA_RxStruct.read + ret) % M_U1_DMA_RX_BUF_SIZE;
  
  return ret;
}

MyBool mU1_TxDmaIsBusy(void)
{
  if(DMA_GetFlagStatus(DMA1_FLAG_TC4) == SET)
  {
    DMA_ClearFlag(DMA1_FLAG_TC4);
    
    mU1_DMA_TxBusy = BOOL_FALSE;
  }
  
  return mU1_DMA_TxBusy;
}

void mUSART1_Config(u32 baudRate)
{
	GPIO_InitTypeDef GPIO_InitStructure;
  USART_InitTypeDef USART_InitStructure;

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_AFIO | RCC_APB2Periph_USART1, ENABLE);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;  //USART1_TX
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;  //USART1_RX
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	USART_InitStructure.USART_BaudRate = baudRate;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
  USART_InitStructure.USART_StopBits = USART_StopBits_1;
  USART_InitStructure.USART_Parity = USART_Parity_No;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
  USART_Init(USART1, &USART_InitStructure);  //USART1
  
  USART_Cmd(USART1, ENABLE);
  USART_DMACmd(USART1, USART_DMAReq_Rx, ENABLE);  //DMAʽ
  USART_DMACmd(USART1, USART_DMAReq_Tx, ENABLE);  //DMAʽ
  USART_ClearFlag(USART1, USART_FLAG_TC); //UARTĴ־λ
//  USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);  //ʹUSART1ж
  USART_ITConfig(USART1, USART_IT_IDLE, ENABLE);  //ʹUSART1 IDLEж
}

void USART1_IRQHandler(void)  //жϺ
{
	u8 clear = clear;
//  if(USART_GetITStatus(USART1, USART_IT_RXNE) != RESET)  //һֽ
//  {
//		USART_ClearITPendingBit(USART1, USART_IT_RXNE);

//  }
	if(USART_GetITStatus(USART1, USART_IT_IDLE) != RESET)  //յһ֡
	{
	  clear = USART1->SR;  //SRĴ
		clear = USART1->DR;  //DRĴ(ȶSRٶDRIDLEж)
    
    mU1_DMA_RxStruct.recv = M_U1_DMA_RX_BUF_SIZE - DMA_GetCurrDataCounter(DMA1_Channel5);
    
    if(networkInterfaceInputHandler == NULL)
    {
      return;
    }

    BaseType_t pxHigherPriorityTaskWoken = pdFALSE;

    vTaskNotifyGiveFromISR(networkInterfaceInputHandler, &pxHigherPriorityTaskWoken);

    portYIELD_FROM_ISR(pxHigherPriorityTaskWoken);
	}
}

void mU1_DMA_RecvInit(void)
{
  DMA_InitTypeDef DMA_Initstructure;
  
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
  DMA_DeInit(DMA1_Channel5);
  
  DMA_Initstructure.DMA_PeripheralBaseAddr = (u32)(&USART1->DR);  /*-治ù-*/
  DMA_Initstructure.DMA_MemoryBaseAddr = (u32)0;  //buf[]
  DMA_Initstructure.DMA_DIR = DMA_DIR_PeripheralSRC;  //赽ڴ
  DMA_Initstructure.DMA_BufferSize = 0;  //len
  DMA_Initstructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_Initstructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
  DMA_Initstructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
  DMA_Initstructure.DMA_MemoryDataSize = DMA_PeripheralDataSize_Byte;
  DMA_Initstructure.DMA_Mode = DMA_Mode_Circular;
  DMA_Initstructure.DMA_Priority = DMA_Priority_High;
  DMA_Initstructure.DMA_M2M = DMA_M2M_Disable;
  DMA_Init(DMA1_Channel5, &DMA_Initstructure);
}

void mU1_DMA_EnRecv(void)
{
  DMA_Cmd(DMA1_Channel5, DISABLE);
  DMA_ClearFlag(DMA1_FLAG_GL5);
  DMA1_Channel5->CMAR = (u32)mU1_DMA_RxStruct.buf;
  DMA_SetCurrDataCounter(DMA1_Channel5, M_U1_DMA_RX_BUF_SIZE);
  DMA_Cmd(DMA1_Channel5, ENABLE);
}

void mU1_DMA_SendInit(void)
{
  DMA_InitTypeDef DMA_Initstructure;
  
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
  DMA_DeInit(DMA1_Channel4);
  
  DMA_Initstructure.DMA_PeripheralBaseAddr = (u32)(&USART1->DR);  /*-治ù-*/
  DMA_Initstructure.DMA_MemoryBaseAddr = (u32)0;  /*-治ù-*/
  DMA_Initstructure.DMA_DIR = DMA_DIR_PeripheralDST;  //ڴ浽
  DMA_Initstructure.DMA_BufferSize = 0;  //=len
  DMA_Initstructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_Initstructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
  DMA_Initstructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
  DMA_Initstructure.DMA_MemoryDataSize = DMA_PeripheralDataSize_Byte;
  DMA_Initstructure.DMA_Mode = DMA_Mode_Normal;
  DMA_Initstructure.DMA_Priority = DMA_Priority_High;
  DMA_Initstructure.DMA_M2M = DMA_M2M_Disable;
  DMA_Init(DMA1_Channel4, &DMA_Initstructure);
}

u16 mU1_DMA_Send(u16 len, u8 buf[])
{
  if(len <= 0)  //ܶDMAͳ0쳣
  {
    return 0;
  }
  
  mU1_DMA_TxBusy = BOOL_TRUE;  //æ״̬
  
  DMA_Cmd(DMA1_Channel4, DISABLE);
  DMA_ClearFlag(DMA1_FLAG_GL4);
  DMA1_Channel4->CMAR = (u32)buf;
  DMA_SetCurrDataCounter(DMA1_Channel4, len);
  DMA_Cmd(DMA1_Channel4, ENABLE);
  
  return len;
}

u16 mU1_BlockSend(u16 len, u8 buf[])
{
  DMA_Cmd(DMA1_Channel4, DISABLE);
  DMA_ClearFlag(DMA1_FLAG_GL4);
  
  for(int i = 0; i < len; i++)
  {
    USART_ClearFlag(USART1, USART_FLAG_TC);
    USART_SendData(USART1, buf[i]);
    while(USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET);
  }
  
  return len;
}













